@@ -17,6 +17,7 @@ from aiocache.backends.redis import RedisCache
 from aiocache.serializers import PickleSerializer
 
 
+from hypha_rpc.rpc import RemoteException
 from hypha import __version__
 from hypha.core import (
     RedisEventBus,
@@ -33,8 +34,8 @@ from sqlalchemy.ext.asyncio import (
 
 from hypha.core.auth import (
     create_scope,
-    parse_token,
-    generate_anonymous_user,
+    parse_auth_token,
+    create_login_service,
     UserPermission,
     AUTH0_CLIENT_ID,
     AUTH0_DOMAIN,
@@ -73,13 +74,21 @@ class WorkspaceInterfaceContextManager:
         return self._get_workspace_manager().__await__()
 
     async def _get_workspace_manager(self):
-        # Check if workspace exists
-        await self._store.load_or_create_workspace(self._user_info, self._workspace)
+        # For the special "public" workspace, do not force creation; allow lightweight access
+        if self._workspace != "public":
+            # Check if workspace exists or create if appropriate
+            await self._store.load_or_create_workspace(self._user_info, self._workspace)
         self._wm = await self._rpc.get_manager_service({"timeout": self._timeout})
         self._wm.rpc = self._rpc
         self._wm.disconnect = self._rpc.disconnect
         self._wm.register_codec = self._rpc.register_codec
         self._wm.register_service = self._rpc.register_service
+        self._wm.unregister_service = self._rpc.unregister_service
+        self._wm.get_service_schema = self._rpc.get_service_schema
+        self._wm.on = self._rpc.on
+        self._wm.off = self._rpc.off
+        self._wm.once = self._rpc.once
+        self._wm.emit = self._rpc.emit
         return self._wm
 
 
@@ -161,14 +170,15 @@ class RedisStore:
         self._enable_service_search = enable_service_search
         self._activity_check_interval = activity_check_interval
         self._enable_s3_for_anonymous_users = enable_s3_for_anonymous_users
-        # Create a fixed HTTP anonymous user
+        # Create a fixed HTTP anonymous user. Grant read on public; set current to public
+        # so we can use it to create a workspace interface safely.
         self._http_anonymous_user = UserInfo(
-            id="http-anonymous",
+            id="anonymouz-http",
             is_anonymous=True,
             email=None,
             parent=None,
             roles=["anonymous"],
-            scope=create_scope("public#r", current_workspace="public"),
+            scope=create_scope("ws-anonymous#r", current_workspace="ws-anonymous"),
             expires_at=None,
         )
         self._anonymous_workspace_info = None
@@ -219,7 +229,7 @@ class RedisStore:
             from redis import asyncio as aioredis
 
             # Configure connection pool for production Redis
-            max_connections = int(os.environ.get("HYPHA_REDIS_MAX_CONNECTIONS", "100"))
+            max_connections = int(os.environ.get("HYPHA_REDIS_MAX_CONNECTIONS", "2000"))
             health_check_interval = int(
                 os.environ.get("HYPHA_REDIS_HEALTH_CHECK_INTERVAL", "30")
             )
@@ -234,6 +244,7 @@ class RedisStore:
                 socket_keepalive_options={},
                 socket_connect_timeout=5,
                 socket_timeout=5,
+                client_name=f"hypha-server:{self._server_id}:cmd",
             )
 
         else:  #  Create a redis server with fakeredis
@@ -291,7 +302,7 @@ class RedisStore:
             email=None,
             parent=None,
             roles=["admin"],
-            scope=create_scope("*#a"),
+            scope=create_scope("*#a", current_workspace="*"),
             expires_at=None,
         )
         return self._root_user
@@ -314,28 +325,42 @@ class RedisStore:
         workspace_info = await self.get_workspace_info(workspace, load=True)
 
         # If workspace does not exist, automatically create it
-        if not workspace_info and workspace == user_info.get_workspace():
-            if not user_info.check_permission(workspace, UserPermission.read):
-                raise PermissionError(
-                    f"User {user_info.id} does not have permission to access workspace {workspace}"
+        if not workspace_info:
+            if workspace == "public":
+                logger.debug("Creating public workspace on demand")
+                workspace_info = await self.register_workspace(
+                    {
+                        "id": "public",
+                        "name": "public",
+                        "description": "Public workspace",
+                        "persistent": True,
+                        "owners": [],
+                        "read_only": False,
+                    }
                 )
-            logger.debug(f"Creating workspace {workspace} for user {user_info.id}")
-            workspace_info = await self.register_workspace(
-                {
-                    "id": workspace,
-                    "name": workspace,
-                    "description": f"Workspace for user {user_info.id}",
-                    "persistent": not user_info.is_anonymous,
-                    "owners": [user_info.id],
-                    "read_only": user_info.is_anonymous
-                    and not self._enable_s3_for_anonymous_users,
-                }
-            )
-        else:
-            if not workspace_info:
+            elif workspace == user_info.get_workspace():
+                if not user_info.check_permission(workspace, UserPermission.read):
+                    raise PermissionError(
+                        f"User {user_info.id} does not have permission to access workspace {workspace}"
+                    )
+                logger.debug(f"Creating workspace {workspace} for user {user_info.id}")
+                workspace_info = await self.register_workspace(
+                    {
+                        "id": workspace,
+                        "name": workspace,
+                        "description": f"Workspace for user {user_info.id}",
+                        "persistent": not user_info.is_anonymous,
+                        "owners": [user_info.id],
+                        "read_only": user_info.is_anonymous
+                        and not self._enable_s3_for_anonymous_users,
+                    }
+                )
+            else:
+                # Workspace doesn't exist and can't be auto-created
                 raise KeyError(
                     f"Workspace {workspace} does not exist or is not accessible"
                 )
+        else:
             logger.debug(f"Workspace {workspace} already exists, skipping creation")
         return workspace_info
 
@@ -598,6 +623,25 @@ class RedisStore:
 
         if startup_functions:
             await self._run_startup_functions(startup_functions)
+        
+        # check if the login service is registered after startup functions
+        # this allows startup functions to register custom login services
+        try:
+            await api.get_service_info("public/hypha-login")
+            logger.info("Login service already registered (likely from startup function)")
+        except RemoteException:
+            logger.info("No custom login service found, registering default login service")
+            await api.register_service(create_login_service(self))
+        
+        # check if the queue service is registered
+        try:
+            await api.get_service_info("public/queue")
+        except RemoteException:
+            logger.warning("Queue service is not registered, registering it now")
+            # Import dynamically to avoid circular import
+            from hypha.queue import create_queue_service
+            await api.register_service(create_queue_service(self))
+        
         self._ready = True
         await self.get_event_bus().emit_local("startup")
         servers = await self.list_servers()
@@ -622,6 +666,7 @@ class RedisStore:
             self._root_user.get_workspace(),
             client_id=self._server_id,
             silent=False,
+            timeout=20,
         )
         await self._root_workspace_interface.register_service(
             {
@@ -635,6 +680,7 @@ class RedisStore:
                 "kickout_client": self.kickout_client,
                 "list_workspaces": self.list_all_workspaces,
                 "unload_workspace": self.unload_workspace,
+                "get_metrics": self.get_metrics,
             }
         )
 
@@ -666,6 +712,14 @@ class RedisStore:
     def get_activity_tracker(self):
         return self._tracker
 
+    @schema_method
+    async def get_metrics(self):
+        """Return internal metrics snapshot for admin observability."""
+        return {
+            "rpc": RedisRPCConnection.get_metrics_snapshot(),
+            "eventbus": RedisEventBus.get_metrics_snapshot(self._event_bus),
+        }
+
     async def get_public_api(self):
         """Get the public API."""
         if self._public_workspace_interface is None:
@@ -717,7 +771,7 @@ class RedisStore:
 
     async def parse_user_token(self, token):
         """Parse a client token."""
-        user_info = parse_token(token)
+        user_info = await parse_auth_token(token)
         key = "revoked_token:" + token
         if await self._redis.exists(key):
             raise Exception("Token has been revoked")
@@ -743,16 +797,15 @@ class RedisStore:
             return user_info
         else:
             # Use a fixed anonymous user
-            if not hasattr(self, "_http_anonymous_user"):
-                self._http_anonymous_user = UserInfo(
-                    id="http-anonymous",
-                    is_anonymous=True,
-                    email=None,
-                    parent=None,
-                    roles=["anonymous"],
-                    scope=create_scope("public#r", current_workspace="public"),
-                    expires_at=None,
-                )
+            self._http_anonymous_user = UserInfo(
+                id="anonymouz-http",  # Use anonymouz- prefix for consistency
+                is_anonymous=True,
+                email=None,
+                parent=None,
+                roles=["anonymous"],
+                scope=create_scope("ws-anonymous#r", current_workspace="ws-anonymous"),
+                expires_at=None,
+            )
             return self._http_anonymous_user
 
     async def get_all_workspace(self):
@@ -796,7 +849,9 @@ class RedisStore:
         )
 
     def get_manager_id(self):
-        return self._workspace_manager.get_client_id()
+        # Return the pre-calculated manager_id directly
+        # This avoids issues during startup when _workspace_manager is not yet initialized
+        return self._manager_id
 
     async def register_workspace_manager(self):
         """Register a workspace manager."""
@@ -814,6 +869,7 @@ class RedisStore:
             self._enable_service_search,
             self._cache_dir,
             self._enable_s3_for_anonymous_users,
+            self._tracker,
         )
         await manager.setup()
         return manager
@@ -924,6 +980,9 @@ class RedisStore:
     def set_artifact_manager(self, controller):
         """Set the artifact controller."""
         self._artifact_manager = controller
+        # Also update the workspace manager's reference if it exists
+        if hasattr(self, '_workspace_manager') and self._workspace_manager:
+            self._workspace_manager._artifact_manager = controller
 
     def register_public_service(self, service: dict):
         """Register a service."""
@@ -1025,6 +1084,14 @@ class RedisStore:
                     pass
 
         await self._event_bus.stop()
+        # Close Redis client gracefully
+        try:
+            if hasattr(self._redis, "close"):
+                await self._redis.close()
+            if hasattr(self._redis, "connection_pool"):
+                await self._redis.connection_pool.disconnect()
+        except Exception:
+            pass
         logger.info("Teardown complete")
 
     @property
@@ -1045,21 +1112,31 @@ class RedisStore:
                 self._store = store
                 self._interface = None
                 self._interface_cm = None
+                self._default_workspace = None  # Cache the workspace to use
 
             def __call__(self, user_info=None, workspace=None):
                 """Support both wm() and wm(context) syntax for context manager."""
+                # If workspace is explicitly provided, remember it for future direct calls
+                if workspace is not None:
+                    self._default_workspace = workspace
+                
+                # Use the cached workspace or default to root user's workspace
+                effective_workspace = workspace or self._default_workspace or "public"
+
                 return WorkspaceManagerContextManager(
                     self._store,
                     user_info or self._store._root_user.model_dump(),
-                    workspace or "public",
+                    effective_workspace,
                 )
 
             async def _ensure_interface(self):
                 """Ensure we have an active interface, creating one if needed."""
                 if self._interface is None:
+                    # Use the cached workspace or default to root user's workspace
+                    workspace = self._default_workspace or "public"
                     self._interface_cm = self._store.get_workspace_interface(
                         self._store._root_user,
-                        "public",
+                        workspace,
                         client_id=None,
                         silent=True,
                     )
