@@ -122,6 +122,26 @@ async def extracted_kwargs(
                 "Invalid content-type (supported types: "
                 "application/msgpack, application/json, text/plain)",
             )
+        # Handle different JSON input types for HTTP service calls
+        if isinstance(kwargs, list):
+            # Convert positional arguments array to a dictionary for internal processing
+            # e.g., ["arg1", "arg2"] becomes {"0": "arg1", "1": "arg2"}
+            kwargs = {str(i): arg for i, arg in enumerate(kwargs)}
+        elif not isinstance(kwargs, dict):
+            # For single primitive values, treat as a single positional argument
+            # e.g., "hello" becomes {"0": "hello"}
+            if isinstance(kwargs, (str, int, float, bool)) or kwargs is None:
+                kwargs = {"0": kwargs}
+            else:
+                raise RuntimeError(
+                    f"Invalid JSON format for HTTP service call. "
+                    f"Expected a dictionary (object), array, or primitive value, but got {type(kwargs).__name__}: {kwargs!r}. "
+                    f"Examples: "
+                    f"Array format: ['arg1', 'arg2'] "
+                    f"Object format: {{'param1': 'value1', 'param2': 'value2'}} "
+                    f"Single value: 'hello'"
+                )
+        
         if use_function_kwargs:
             kwargs = kwargs.get("function_kwargs", {})
         else:
@@ -242,6 +262,8 @@ class ASGIRoutingMiddleware:
                         authorization=authorization, access_token=access_token
                     )
 
+                    # Here we must always use the user's current workspace for the interface
+                    # This is a security measure to prevent unauthorized access to private workspaces
                     async with self.store.get_workspace_interface(
                         user_info, user_info.scope.current_workspace
                     ) as api:
@@ -548,6 +570,13 @@ class HTTPProxy:
                 )
 
         @app.get(norm_url("/{workspace}/services"))
+        async def get_workspace_services_redirect(
+            workspace: str,
+            user_info: store.login_optional = Depends(store.login_optional),
+        ):
+            return RedirectResponse(url=norm_url(f"/{workspace}/services/"))
+
+        @app.get(norm_url("/{workspace}/services/"))
         async def get_workspace_services(
             workspace: str,
             user_info: store.login_optional = Depends(store.login_optional),
@@ -882,6 +911,9 @@ class HTTPProxy:
             """
             try:
                 workspace, service_id, function_key = function_info
+                # The workspace should always be the user's current workspace
+                # derived from the token
+                # We should never trust the workspace specified in the url or query
                 async with self.store.get_workspace_interface(
                     user_info, user_info.scope.current_workspace
                 ) as api:
@@ -1113,6 +1145,11 @@ class HTTPProxy:
                             "detail": f"File not found: {inner_path}",
                         },
                     )
+                else:
+                    # redirect /{workspace} to /{workspace}/
+                    if not page.endswith("/") and inner_path == "index.html":
+                        # redirect it to the version with trailing slash
+                        return RedirectResponse(norm_url(f"/{dir_path}/"))
                 return FileResponse(safe_join(str(self.ws_apps_dir), inner_path))
             file_path = safe_join(str(self.templates_dir), page)
             if not is_safe_path(str(self.templates_dir), file_path):
