@@ -15,9 +15,7 @@ from fastapi.responses import FileResponse
 from fastapi.staticfiles import StaticFiles
 
 from hypha import __version__
-from hypha.core.auth import create_login_service
 from hypha.core.store import RedisStore
-from hypha.queue import create_queue_service
 from hypha.http import HTTPProxy
 from hypha.triton import TritonProxy
 from hypha.utils import GZipMiddleware, GzipRoute, PatchedCORSMiddleware
@@ -88,8 +86,9 @@ def start_builtin_services(
             allow_origins=args.allow_origins,
         )
 
-    store.register_public_service(create_queue_service(store))
-    store.register_public_service(create_login_service(store))
+    # Initialize variables to None first
+    s3_controller = None
+    artifact_manager = None
 
     if args.enable_s3:
         # pylint: disable=import-outside-toplevel
@@ -128,6 +127,7 @@ def start_builtin_services(
             port=args.port,
             in_docker=args.in_docker,
             artifact_manager=artifact_manager,
+            disable_ssl=args.disable_ssl,
         )
 
     HTTPProxy(
@@ -144,7 +144,7 @@ def start_builtin_services(
     )
 
 
-def mount_static_files(app, new_route, directory, name="static"):
+def mount_static_files(app, base_path, new_route, directory, name="static"):
     # Get top level route paths
     top_level_route_paths = [
         route.path.split("/")[1] for route in app.routes if route.path.count("/") == 1
@@ -164,11 +164,11 @@ def mount_static_files(app, new_route, directory, name="static"):
         raise FileNotFoundError(f"The directory '{directory}' does not exist.")
 
     # If no collision, mount static files
-    app.mount(new_route, StaticFiles(directory=directory), name=name)
+    app.mount(norm_url(base_path, new_route), StaticFiles(directory=directory), name=name)
 
     if Path(f"{directory}/index.html").exists():
         # Add a new route that serves index.html directly
-        @app.get(new_route)
+        @app.get(norm_url(base_path, new_route))
         async def root():
             return FileResponse(f"{directory}/index.html")
 
@@ -243,6 +243,27 @@ def create_application(args):
             args.startup_functions.append(a2a_startup_function)
             logger.info("Automatically added A2A proxy worker to startup functions")
 
+    # Automatically add K8s worker startup function if K8s worker is enabled
+    if args.enable_k8s_worker:
+        k8s_startup_function = "hypha.workers.k8s:hypha_startup"
+        if k8s_startup_function not in args.startup_functions:
+            args.startup_functions.append(k8s_startup_function)
+            logger.info("Automatically added Kubernetes worker to startup functions")
+
+    # Automatically add Terminal worker startup function if Terminal worker is enabled
+    if args.enable_terminal_worker:
+        terminal_startup_function = "hypha.workers.terminal:hypha_startup"
+        if terminal_startup_function not in args.startup_functions:
+            args.startup_functions.append(terminal_startup_function)
+            logger.info("Automatically added Terminal worker to startup functions")
+    
+    # Automatically add Local Auth startup function if Local Auth is enabled
+    if args.enable_local_auth:
+        local_auth_startup_function = "hypha.local_auth:hypha_startup"
+        if local_auth_startup_function not in args.startup_functions:
+            args.startup_functions.append(local_auth_startup_function)
+            logger.info("Automatically added Local Authentication provider to startup functions")
+
     @asynccontextmanager
     async def lifespan(app: FastAPI):
         # Here we can register all the startup functions
@@ -324,13 +345,13 @@ def create_application(args):
     websocket_server = WebsocketServer(store, path=norm_url(args.base_path, "/ws"))
 
     static_folder = str(Path(__file__).parent / "static_files")
-    mount_static_files(application, "/static", directory=static_folder, name="static")
+    mount_static_files(application, args.base_path, "/static", directory=static_folder, name="static")
 
     if args.static_mounts:
         for index, mount in enumerate(args.static_mounts):
             mountpath, localdir = mount.split(":")
             mount_static_files(
-                application, mountpath, localdir, name=f"static-mount-{index}"
+                application, args.base_path,mountpath, localdir, name=f"static-mount-{index}"
             )
 
     start_builtin_services(application, store, args)
@@ -585,6 +606,21 @@ def get_argparser(add_help=True):
         action="store_true",
         help="enable MCP (Model Context Protocol) support",
     )
+    parser.add_argument(
+        "--enable-k8s-worker",
+        action="store_true",
+        help="enable Kubernetes pod worker support",
+    )
+    parser.add_argument(
+        "--enable-terminal-worker",
+        action="store_true",
+        help="enable Terminal worker support for executing commands",
+    )
+    parser.add_argument(
+        "--enable-local-auth",
+        action="store_true",
+        help="enable local authentication provider with user management",
+    )
     parser.add_argument(
         "--interactive",
         action="store_true",
@@ -652,6 +688,11 @@ def get_argparser(add_help=True):
         action="store_true",
         help="allow anonymous users to use S3 and artifact functionality (removes read-only restriction)",
     )
+    parser.add_argument(
+        "--disable-ssl",
+        action="store_true",
+        help="disable SSL verification",
+    )
     return parser
 
 
