@@ -2,6 +2,7 @@ import logging
 import os
 import sys
 import json
+import msgpack
 
 from fastapi import Query, WebSocket, status
 from starlette.websockets import WebSocketDisconnect
@@ -12,7 +13,7 @@ from hypha import __version__
 from hypha.core import UserInfo, UserPermission
 from hypha.core.store import RedisRPCConnection, RedisStore
 from hypha.core.auth import (
-    generate_reconnection_token,
+    generate_auth_token,
     generate_anonymous_user,
     create_scope,
     update_user_scope,
@@ -85,13 +86,17 @@ class WebsocketServer:
                     )
                 else:
                     user_info = generate_anonymous_user()
+                    # Each anonymous user gets their own workspace based on their ID
+                    user_workspace = user_info.get_workspace()
+                    # Use the requested workspace if specified, otherwise use the user's own workspace
+                    workspace = workspace or user_workspace
+                    # Anonymous users get admin permission for their own workspace
                     user_info.scope = create_scope(
-                        current_workspace=user_info.get_workspace(),
-                        workspaces={user_info.get_workspace(): UserPermission.admin},
+                        current_workspace=workspace,
+                        workspaces={user_workspace: UserPermission.admin},
                         client_id=client_id,
                     )
-                    workspace = workspace or user_info.get_workspace()
-                    logger.info(f"Created anonymous user {user_info.id}")
+                    logger.info(f"Created anonymous user {user_info.id} with workspace {user_workspace}")
 
                 workspace_info = await self.store.load_or_create_workspace(
                     user_info, workspace
@@ -131,6 +136,7 @@ class WebsocketServer:
                 )
             except RuntimeError as exp:
                 # this happens when the websocket is closed
+                logger.error(f"RuntimeError in establish_websocket_communication: {str(exp)}")
                 reason = f"WebSocket runtime error: {str(exp)}"
                 await self.handle_disconnection(
                     websocket,
@@ -255,7 +261,7 @@ class WebsocketServer:
                 if user_info.scope.current_workspace and workspace:
                     assert (
                         workspace == user_info.scope.current_workspace
-                    ), f"Current workspace encoded in the token ({user_info.scope.current_workspace}) does not the specified workspace ({workspace})"
+                    ), f"Current workspace encoded in the token ({user_info.scope.current_workspace}) does not match the specified workspace ({workspace})"
                 if not workspace and user_info.scope.current_workspace:
                     workspace = user_info.scope.current_workspace
             else:
@@ -285,8 +291,13 @@ class WebsocketServer:
         event_bus = self.store.get_event_bus()
         assert (
             user_info.scope.current_workspace == workspace
-        ), f"Workspace mismatch: {workspace} != {user_info.current_workspace}"
-        conn = RedisRPCConnection(event_bus, workspace, client_id, user_info, None)
+        ), f"Workspace mismatch: {workspace} != {user_info.scope.current_workspace}"
+        
+        # Determine if client has read-only permissions
+        user_permission = user_info.get_permission(workspace)
+        is_readonly = user_permission == UserPermission.read if user_permission else False
+        
+        conn = RedisRPCConnection(event_bus, workspace, client_id, user_info, self.store.get_manager_id(), readonly=is_readonly)
         self._websockets[f"{workspace}/{client_id}"] = websocket
         try:
             _gauge.labels(workspace=workspace).inc()
@@ -294,12 +305,17 @@ class WebsocketServer:
 
             async def send_bytes(data):
                 try:
+                    # Handle both bytes and dict data
+                    if isinstance(data, dict):
+                        data = msgpack.packb(data)
+                    elif isinstance(data, str):
+                        data = data.encode('utf-8')
                     await websocket.send_bytes(data)
                 except Exception as e:
                     logger.error("Failed to send message via websocket: %s", str(e))
 
             conn.on_message(send_bytes)
-            reconnection_token = generate_reconnection_token(
+            reconnection_token = await generate_auth_token(
                 user_info, expires_in=self.store.reconnection_token_life_time
             )
             conn_info = {
@@ -332,7 +348,7 @@ class WebsocketServer:
                         if data.get("type") == "ping":
                             await websocket.send_text(json.dumps({"type": "pong"}))
                         elif data.get("type") == "refresh_token":
-                            reconnection_token = generate_reconnection_token(
+                            reconnection_token = await generate_auth_token(
                                 user_info,
                                 expires_in=self.store.reconnection_token_life_time,
                             )
