@@ -19,6 +19,7 @@ from hypha.workers.base import (
     SessionInfo,
     SessionNotFoundError,
     WorkerError,
+    safe_call_callback,
 )
 
 LOGLEVEL = os.environ.get("HYPHA_LOGLEVEL", "INFO").upper()
@@ -28,7 +29,8 @@ logger.setLevel(LOGLEVEL)
 
 # Try to import A2A SDK
 try:
-    from a2a.client.client import A2AClient, A2ACardResolver
+    from a2a.types import AgentCard
+    from a2a.client import A2AClient, A2ACardResolver
     from a2a.types import SendMessageRequest, MessageSendParams, Message, TextPart, Role
 
     A2A_SDK_AVAILABLE = True
@@ -72,6 +74,11 @@ class A2AClientRunner(BaseWorker):
         """Return whether the worker requires a context."""
         return True
 
+    @property
+    def use_local_url(self) -> bool:
+        """Return whether the worker should use local URLs."""
+        return True  # Built-in worker runs in same cluster/host
+
     async def compile(
         self,
         manifest: dict,
@@ -195,6 +202,7 @@ class A2AClientRunner(BaseWorker):
             logger.error(f"Failed to start A2A session {session_id}: {e}")
             # Clean up failed session
             self._sessions.pop(session_id, None)
+            self._session_data.pop(session_id, None)  # Also clean up session data if it exists
             raise
 
     async def _start_a2a_session(self, config: WorkerConfig) -> Dict[str, Any]:
@@ -203,7 +211,7 @@ class A2AClientRunner(BaseWorker):
             raise Exception("A2A SDK not available. Install with: pip install a2a")
 
         # Call progress callback if provided
-        config.progress_callback(
+        await safe_call_callback(config.progress_callback,
             {"type": "info", "message": "Initializing A2A proxy worker..."}
         )
 
@@ -221,7 +229,7 @@ class A2AClientRunner(BaseWorker):
             "services": [],
         }
 
-        config.progress_callback(
+        await safe_call_callback(config.progress_callback,
             {"type": "info", "message": "Connecting to Hypha server..."}
         )
 
@@ -234,7 +242,7 @@ class A2AClientRunner(BaseWorker):
             }
         )
 
-        config.progress_callback(
+        await safe_call_callback(config.progress_callback,
             {
                 "type": "info",
                 "message": f"Connecting to {len(a2a_agents)} A2A agent(s)...",
@@ -244,7 +252,7 @@ class A2AClientRunner(BaseWorker):
         # Connect to each A2A agent
         for agent_name, agent_config in a2a_agents.items():
             try:
-                config.progress_callback(
+                await safe_call_callback(config.progress_callback,
                     {
                         "type": "info",
                         "message": f"Connecting to A2A agent: {agent_name}...",
@@ -260,7 +268,7 @@ class A2AClientRunner(BaseWorker):
                     f"Successfully connected to A2A agent: {agent_name}"
                 )
                 logger.info(f"Successfully connected to A2A agent: {agent_name}")
-                config.progress_callback(
+                await safe_call_callback(config.progress_callback,
                     {
                         "type": "info",
                         "message": f"Successfully connected to A2A agent: {agent_name}",
@@ -271,14 +279,14 @@ class A2AClientRunner(BaseWorker):
                 error_msg = f"Failed to connect to A2A agent {agent_name}: {str(e)}"
                 session_data["logs"]["error"].append(error_msg)
                 logger.error(error_msg)
-                config.progress_callback({"type": "error", "message": error_msg})
+                await safe_call_callback(config.progress_callback, {"type": "error", "message": error_msg})
                 # For single agent configs, fail fast instead of continuing
                 if len(a2a_agents) == 1:
                     raise Exception(error_msg)
                 # Continue with other agents even if one fails
                 continue
 
-        config.progress_callback(
+        await safe_call_callback(config.progress_callback,
             {"type": "info", "message": "Registering default service..."}
         )
 
@@ -300,14 +308,14 @@ class A2AClientRunner(BaseWorker):
                 error_msg = (
                     f"Failed to connect to any A2A agents. Last error: {last_error}"
                 )
-                config.progress_callback({"type": "error", "message": error_msg})
+                await safe_call_callback(config.progress_callback, {"type": "error", "message": error_msg})
                 raise Exception(error_msg)
             else:
                 error_msg = "Failed to connect to any A2A agents"
-                config.progress_callback({"type": "error", "message": error_msg})
+                await safe_call_callback(config.progress_callback, {"type": "error", "message": error_msg})
                 raise Exception(error_msg)
 
-        config.progress_callback(
+        await safe_call_callback(config.progress_callback,
             {
                 "type": "success",
                 "message": f"A2A proxy initialized successfully with {len(session_data['a2a_clients'])} agent(s)",
@@ -337,7 +345,7 @@ class A2AClientRunner(BaseWorker):
             logger.info(f"Connecting to A2A agent {agent_name} at {agent_url}")
 
             if config:
-                config.progress_callback(
+                await safe_call_callback(config.progress_callback,
                     {
                         "type": "info",
                         "message": f"Discovering capabilities from {agent_name}...",
@@ -356,14 +364,14 @@ class A2AClientRunner(BaseWorker):
             }
 
             # Resolve agent card and create A2A client
-            agent_card_url = f"{agent_url}/.well-known/agent.json"
+            agent_card_url = f"{agent_url}/.well-known/agent-card.json"
             resolver = A2ACardResolver(
                 httpx_client=http_client,
                 base_url=agent_url,
-                agent_card_path="/.well-known/agent.json",
+                agent_card_path="/.well-known/agent-card.json",
             )
 
-            agent_card = await resolver.get_agent_card()
+            agent_card: AgentCard = await resolver.get_agent_card()
             if not agent_card:
                 raise ValueError(f"Failed to resolve agent card for {agent_name}")
 
@@ -378,13 +386,13 @@ class A2AClientRunner(BaseWorker):
             logger.info(f"Agent {agent_name}: Found {len(skills)} skills")
 
             if config:
-                config.progress_callback(
+                await safe_call_callback(config.progress_callback,
                     {
                         "type": "info",
                         "message": f"Found {len(skills)} skills from {agent_name}",
                     }
                 )
-                config.progress_callback(
+                await safe_call_callback(config.progress_callback,
                     {
                         "type": "info",
                         "message": f"Registering services for {agent_name}...",
@@ -518,7 +526,7 @@ class A2AClientRunner(BaseWorker):
         return skill_wrappers
 
     async def _register_unified_a2a_service(
-        self, session_data: dict, agent_name: str, agent_card, skills: List, client
+        self, session_data: dict, agent_name: str, agent_card: AgentCard, skills: List, client
     ):
         """Register a unified A2A service with skills."""
         try:
@@ -553,29 +561,30 @@ class A2AClientRunner(BaseWorker):
                     "capabilities": (
                         {
                             "streaming": bool(agent_card.capabilities.streaming),
-                            "pushNotifications": bool(
-                                agent_card.capabilities.pushNotifications
+                                                    "push_notifications": bool(
+                            agent_card.capabilities.push_notifications
                             ),
                         }
                         if agent_card.capabilities
                         else {}
                     ),
-                    "defaultInputModes": (
-                        [str(mode) for mode in agent_card.defaultInputModes]
-                        if agent_card.defaultInputModes
+                                    "default_input_modes": (
+                    [str(mode) for mode in agent_card.default_input_modes]
+                    if agent_card.default_input_modes
                         else []
                     ),
-                    "defaultOutputModes": (
-                        [str(mode) for mode in agent_card.defaultOutputModes]
-                        if agent_card.defaultOutputModes
+                    "default_output_modes": (
+                        [str(mode) for mode in agent_card.default_output_modes]
+                        if agent_card.default_output_modes
                         else []
                     ),
                 },
             }
 
             # Register the service
-            await client.register_service(service_def, overwrite=True)
-            session_data["services"].append(service_def["name"])
+            service_info = await client.register_service(service_def, overwrite=True)
+            # Store the actual service ID that was returned by register_service
+            session_data["services"].append(service_info.id)
             logger.info(
                 f"Registered unified A2A service: {service_def['name']} with ID: {agent_name}"
             )
@@ -591,10 +600,7 @@ class A2AClientRunner(BaseWorker):
     ) -> None:
         """Stop an A2A session."""
         if session_id not in self._sessions:
-            logger.warning(
-                f"A2A session {session_id} not found for stopping, may have already been cleaned up"
-            )
-            return
+            raise SessionNotFoundError(f"A2A session {session_id} not found for stopping")
 
         session_info = self._sessions[session_id]
         session_info.status = SessionStatus.STOPPING
@@ -602,34 +608,36 @@ class A2AClientRunner(BaseWorker):
         try:
             session_data = self._session_data.get(session_id)
             if session_data:
-                # Get the client from session data, not self.server
+                # Get the client from session data
                 client = session_data.get("client")
-                
-                # Unregister services
-                services = session_data.get("services", [])
-                for service_name in services:
-                    try:
-                        if client:
+                if client:
+                    # Unregister services
+                    services = session_data.get("services", [])
+                    for service_name in services:
+                        try:
                             await client.unregister_service(service_name)
                             logger.info(f"Unregistered service: {service_name}")
-                        else:
-                            logger.warning(f"No client available to unregister service {service_name}")
+                        except Exception as e:
+                            logger.warning(f"Failed to unregister service {service_name}: {e}")
+
+                    # Disconnect the Hypha client to prevent memory leak
+                    try:
+                        await client.disconnect()
+                        logger.info(f"Disconnected Hypha client for session {session_id}")
                     except Exception as e:
-                        logger.warning(
-                            f"Error unregistering service {service_name}: {e}"
-                        )
+                        logger.warning(f"Failed to disconnect client for session {session_id}: {e}")
 
                 # Close all HTTP clients
                 a2a_clients = session_data.get("a2a_clients", {})
                 for agent_name, agent_info in a2a_clients.items():
-                    try:
-                        if "client" in agent_info:
+                    if "client" in agent_info:
+                        try:
                             await agent_info["client"].aclose()
                             logger.info(f"Closed HTTP client for agent: {agent_name}")
-                    except Exception as e:
-                        logger.warning(
-                            f"Failed to close HTTP client for agent {agent_name}: {e}"
-                        )
+                        except Exception as e:
+                            logger.warning(f"Failed to close HTTP client for {agent_name}: {e}")
+            else:
+                logger.warning(f"No session data found for session {session_id}, skipping cleanup")
 
             session_info.status = SessionStatus.STOPPED
             logger.info(f"Stopped A2A session {session_id}")
@@ -644,23 +652,6 @@ class A2AClientRunner(BaseWorker):
             self._sessions.pop(session_id, None)
             self._session_data.pop(session_id, None)
 
-    async def list_sessions(
-        self, workspace: str, context: Optional[Dict[str, Any]] = None
-    ) -> List[SessionInfo]:
-        """List all A2A sessions for a workspace."""
-        return [
-            session_info
-            for session_info in self._sessions.values()
-            if session_info.workspace == workspace
-        ]
-
-    async def get_session_info(
-        self, session_id: str, context: Optional[Dict[str, Any]] = None
-    ) -> SessionInfo:
-        """Get information about an A2A session."""
-        if session_id not in self._sessions:
-            raise SessionNotFoundError(f"A2A session {session_id} not found")
-        return self._sessions[session_id]
 
     async def get_logs(
         self,
@@ -669,73 +660,61 @@ class A2AClientRunner(BaseWorker):
         offset: int = 0,
         limit: Optional[int] = None,
         context: Optional[Dict[str, Any]] = None,
-    ) -> Union[Dict[str, List[str]], List[str]]:
-        """Get logs for an A2A session."""
+    ) -> Dict[str, Any]:
+        """Get logs for an A2A session.
+        
+        Returns a dictionary with:
+        - items: List of log events, each with 'type' and 'content' fields
+        - total: Total number of log items (before filtering/pagination)
+        - offset: The offset used for pagination
+        - limit: The limit used for pagination
+        """
         if session_id not in self._sessions:
             raise SessionNotFoundError(f"A2A session {session_id} not found")
 
         session_data = self._session_data.get(session_id)
         if not session_data:
-            return {} if type is None else []
+            return {"items": [], "total": 0, "offset": offset, "limit": limit}
 
         logs = session_data.get("logs", {})
-
+        
+        # Convert logs to items format
+        all_items = []
+        for log_type, log_entries in logs.items():
+            for entry in log_entries:
+                all_items.append({"type": log_type, "content": entry})
+        
+        # Filter by type if specified
         if type:
-            target_logs = logs.get(type, [])
-            end_idx = (
-                len(target_logs)
-                if limit is None
-                else min(offset + limit, len(target_logs))
-            )
-            return target_logs[offset:end_idx]
+            filtered_items = [item for item in all_items if item["type"] == type]
         else:
-            result = {}
-            for log_type_key, log_entries in logs.items():
-                end_idx = (
-                    len(log_entries)
-                    if limit is None
-                    else min(offset + limit, len(log_entries))
-                )
-                result[log_type_key] = log_entries[offset:end_idx]
-            return result
+            filtered_items = all_items
+        
+        total = len(filtered_items)
+        
+        # Apply pagination
+        if limit is None:
+            paginated_items = filtered_items[offset:]
+        else:
+            paginated_items = filtered_items[offset:offset + limit]
+        
+        return {
+            "items": paginated_items,
+            "total": total,
+            "offset": offset,
+            "limit": limit
+        }
 
-    async def prepare_workspace(
-        self, workspace: str, context: Optional[Dict[str, Any]] = None
-    ) -> None:
-        """Prepare workspace for A2A operations."""
-        logger.info(f"Preparing workspace {workspace} for A2A proxy worker")
-        pass
-
-    async def close_workspace(
-        self, workspace: str, context: Optional[Dict[str, Any]] = None
-    ) -> None:
-        """Close all A2A sessions for a workspace."""
-        logger.info(f"Closing workspace {workspace} for A2A proxy worker")
-
-        # Stop all sessions for this workspace
-        sessions_to_stop = [
-            session_id
-            for session_id, session_info in self._sessions.items()
-            if session_info.workspace == workspace
-        ]
-
-        for session_id in sessions_to_stop:
-            try:
-                await self.stop(session_id)
-            except Exception as e:
-                logger.warning(f"Failed to stop A2A session {session_id}: {e}")
+    
 
     async def shutdown(self, context: Optional[Dict[str, Any]] = None) -> None:
         """Shutdown the A2A proxy worker."""
         logger.info("Shutting down A2A proxy worker...")
 
-        # Stop all sessions
+        # Stop all sessions - any failure should be propagated
         session_ids = list(self._sessions.keys())
         for session_id in session_ids:
-            try:
-                await self.stop(session_id)
-            except Exception as e:
-                logger.warning(f"Failed to stop A2A session {session_id}: {e}")
+            await self.stop(session_id)
 
         logger.info("A2A proxy worker shutdown complete")
 
