@@ -1,26 +1,28 @@
 """Provide an example plugin."""
-from imjoy_rpc import api
+
+from hypha_rpc import api
 
 # pylint: disable=no-self-use
 
 
-class ImJoyPlugin:
-    """Represent an ImJoy plugin."""
+class HyphaApp:
+    """Represent an Hypha app."""
 
     async def setup(self):
         """Set up the plugin."""
-        token = await api.generateToken()
-        assert "@imjoy@" in token
-        print(f"Generated token: {token}")
+        message = await api.echo("This is a test")
+        print(message)
 
         service_info = await api.register_service(
             {
+                "id": "test-service",
                 "name": "echo service",
                 "type": "echo",
                 "echo": lambda x: print("echo: " + str(x)),
-            }
+            },
+            {"overwrite": True},
         )
-        service = await api.get_service(service_info)
+        service = await api.get_service(service_info.id)
         await service.echo("a message")
         await api.log("initialized")
 
@@ -29,4 +31,4 @@ class ImJoyPlugin:
         await api.log("hello world")
 
 
-api.export(ImJoyPlugin(), config={"name": "test-plugin", "workspace": "123"})
+api.export(HyphaApp(), config={"name": "test-plugin"})
