@@ -134,18 +134,17 @@ async def test_conda_worker_service_registration(fastapi_server, test_user_token
     assert worker_service is not None
     assert hasattr(worker_service, "start")
     assert hasattr(worker_service, "stop")
-    assert hasattr(worker_service, "execute_code")
+    assert hasattr(worker_service, "execute")
     assert hasattr(worker_service, "get_logs")
-    assert hasattr(worker_service, "list_sessions")
 
     print("✓ Conda worker registered successfully")
 
     # Test supported types
-    assert "python-conda" in worker.supported_types
+    assert "conda-jupyter-kernel" in worker.supported_types
     print(f"✓ Conda worker supports types: {worker.supported_types}")
 
     # Test worker info
-    assert "Conda Environment Worker" in worker.name
+    assert "Conda Worker" in worker.name
     assert worker.description is not None
     print(f"✓ Worker info: {worker.name}")
 
@@ -185,9 +184,7 @@ async def test_browser_worker_service_registration(fastapi_server, test_user_tok
     assert worker_service is not None
     assert hasattr(worker_service, "start")
     assert hasattr(worker_service, "stop")
-    assert hasattr(worker_service, "take_screenshot")
     assert hasattr(worker_service, "get_logs")
-    assert hasattr(worker_service, "list_sessions")
     assert hasattr(worker_service, "compile")
 
     print("✓ Browser worker registered successfully")
@@ -241,36 +238,10 @@ async def test_conda_worker_session_lifecycle(fastapi_server, test_user_token):
     await api.rpc.register_service(service_config)
     worker_service = await api.get_service("test-conda-session-worker")
 
-    # Test session lifecycle
-    # Create session config
-    session_config = {
-        "id": f"{workspace}/test-conda-session",
-        "client_id": "test-conda-session",
-        "app_id": "test-conda-app",
-        "server_url": WS_SERVER_URL,
-        "workspace": workspace,
-        "token": test_user_token,
-        "entry_point": "main.py",
-        "artifact_id": f"{workspace}/test-conda-app",
-        "manifest": {
-            "type": "python-conda",
-            "name": "Test Conda App",
-            "entry_point": "main.py",
-            "dependencies": ["python=3.11", "numpy"],
-            "channels": ["conda-forge"],
-        },
-        "app_files_base_url": f"{SERVER_URL}/{workspace}/artifacts/test-conda-app/files",
-        "progress_callback": lambda x: print(f"Progress: {x}"),
-    }
 
     # Note: We can't actually start a conda session without proper setup
     # So we'll test the session management methods directly
 
-    # Test list sessions (should be empty initially)
-    sessions = await worker_service.list_sessions(workspace)
-    assert isinstance(sessions, list)
-    print("✓ List sessions works")
-
     # Test get logs for non-existent session (should handle gracefully)
     try:
         logs = await worker_service.get_logs("non-existent-session")
@@ -313,12 +284,6 @@ async def test_browser_worker_session_lifecycle(fastapi_server, test_user_token)
     await api.rpc.register_service(service_config)
     worker_service = await api.get_service("test-browser-session-worker")
 
-    # Test session lifecycle
-    # Test list sessions (should be empty initially)
-    sessions = await worker_service.list_sessions(workspace)
-    assert isinstance(sessions, list)
-    print("✓ List sessions works")
-
     # Test get logs for non-existent session (should handle gracefully)
     try:
         logs = await worker_service.get_logs("non-existent-session")
@@ -369,27 +334,29 @@ async def test_browser_worker_session_lifecycle(fastapi_server, test_user_token)
 
 async def test_conda_worker_cli_functionality():
     """Test conda worker CLI argument parsing and configuration."""
-    from hypha.workers.conda import main
+    import subprocess
     import sys
-    from unittest.mock import patch
 
     # Test help functionality (should not raise exception)
-    with patch.object(sys, "argv", ["conda.py", "--help"]):
-        try:
-            main()
-        except SystemExit as e:
-            # --help causes SystemExit with code 0
-            assert e.code == 0
+    result = subprocess.run(
+        [sys.executable, "-m", "hypha.workers.conda", "--help"],
+        capture_output=True,
+        text=True
+    )
+    assert result.returncode == 0
+    assert "Hypha Conda Environment Worker" in result.stdout
 
     print("✓ Conda worker CLI help works")
 
-    # Test missing required arguments
-    with patch.object(sys, "argv", ["conda.py"]):
-        try:
-            main()
-        except SystemExit as e:
-            # Missing required args should cause SystemExit with code 1
-            assert e.code == 1
+    # Test missing required arguments (should show help and environment variables info)
+    result = subprocess.run(
+        [sys.executable, "-m", "hypha.workers.conda"],
+        capture_output=True,
+        text=True,
+        env={}  # Clear environment to ensure no HYPHA_ variables are set
+    )
+    # Without required arguments or env vars, it should fail
+    assert result.returncode != 0 or "Server URL" in result.stdout
 
     print("✓ Conda worker CLI validates required arguments")
 
@@ -697,10 +664,11 @@ print("Python worker test app registered successfully")
     # Check logs - Python worker outputs to stdout, not log
     logs = await controller.get_logs(config.id)
 
-    # Get all log content from different log types (stdout, stderr, log, etc.)
+    # Get all log content from log items
     all_logs = []
-    for log_type, entries in logs.items():
-        all_logs.extend(entries)
+    if "items" in logs:
+        for item in logs["items"]:
+            all_logs.append(item["content"])
     full_log_text = " ".join(all_logs)
 
     # Check for the expected output in any log type
@@ -742,7 +710,7 @@ async def test_mcp_worker_integration(fastapi_server, test_user_token):
             "name": "Test MCP Service",
             "type": "mcp",
             "config": {"visibility": "public"},
-            "tools": [test_tool],
+            "tools": {"test_tool": test_tool},
         }
     )
 
@@ -825,8 +793,8 @@ async def test_a2a_worker_integration(fastapi_server, test_user_token):
                 "version": "1.0.0",
                 "url": f"{SERVER_URL}/{workspace}/a2a/test-a2a-service",
                 "capabilities": {"streaming": False},
-                "defaultInputModes": ["text/plain"],
-                "defaultOutputModes": ["text/plain"],
+                "default_input_modes": ["text/plain"],
+                "default_output_modes": ["text/plain"],
             },
         }
     )
@@ -916,12 +884,22 @@ async def test_workspace_worker_registration(fastapi_server, test_user_token):
         session_id: str, type: str = None, offset: int = 0, limit: int = None
     ) -> dict:
         """Get logs for a custom worker session."""
-        return {"log": [f"Custom worker log for {session_id}"], "error": []}
+        items = [{"type": "log", "content": f"Custom worker log for {session_id}"}]
+        if type:
+            items = [item for item in items if item["type"] == type]
+        
+        if limit is not None:
+            items = items[offset:offset + limit]
+        else:
+            items = items[offset:]
+        
+        return {
+            "items": items,
+            "total": 1,
+            "offset": offset,
+            "limit": limit
+        }
 
-    @schema_function
-    async def list_sessions(workspace: str) -> list:
-        """List sessions for a workspace."""
-        return []
 
     # Register the custom worker
     worker_service = await api.register_service(
@@ -937,7 +915,6 @@ async def test_workspace_worker_registration(fastapi_server, test_user_token):
             "start": start,
             "stop": stop,
             "get_logs": get_logs,
-            "list_sessions": list_sessions,
         }
     )
 
@@ -953,7 +930,6 @@ async def test_workspace_worker_registration(fastapi_server, test_user_token):
     assert hasattr(worker, "start")
     assert hasattr(worker, "stop")
     assert hasattr(worker, "get_logs")
-    assert hasattr(worker, "list_sessions")
     print("✓ Worker methods are accessible")
 
     # Test calling the worker methods directly
@@ -974,18 +950,16 @@ async def test_workspace_worker_registration(fastapi_server, test_user_token):
 
     # Test getting logs
     logs = await worker.get_logs(session_data["session_id"])
-    assert "Custom worker log" in logs["log"][0]
+    assert "items" in logs
+    log_items = [item for item in logs["items"] if item["type"] == "log"]
+    assert len(log_items) > 0
+    assert "Custom worker log" in log_items[0]["content"]
     print("✓ Worker get_logs method works correctly")
 
     # Test stopping session
     await worker.stop(session_data["session_id"])
     print("✓ Worker stop method works correctly")
 
-    # Test listing sessions
-    sessions = await worker.list_sessions(workspace)
-    assert isinstance(sessions, list)
-    print("✓ Worker list_sessions method works correctly")
-
     # Clean up
     await api.unregister_service(worker_service["id"])
     print("✓ Custom worker integration test passed")
@@ -1240,8 +1214,9 @@ async def test_worker_lifecycle_management(fastapi_server, test_user_token):
 
     # 4. Get logs
     logs = await controller.get_logs(session_info["id"])
-    assert "log" in logs
-    assert len(logs["log"]) > 0
+    assert "items" in logs
+    console_logs = [item for item in logs["items"] if item["type"] == "console"]
+    assert len(console_logs) > 0
     print("✓ Logs retrieved successfully")
 
     # 5. Stop app
@@ -1363,5 +1338,243 @@ async def test_worker_workspace_isolation(
     await api2.disconnect()
 
 
+@pytest.mark.asyncio
+async def test_worker_death_session_cleanup(fastapi_server, test_user_token):
+    """Test that sessions are properly cleaned up when a worker dies."""
+    from hypha.utils import random_id
+    from hypha.workers.base import BaseWorker, WorkerConfig, SessionInfo, SessionStatus
+    
+    # Connect as the main test client
+    api = await connect_to_server(
+        {"client_id": "test-worker-death-main", "server_url": SERVER_URL, "token": test_user_token}
+    )
+    
+    # Get the app controller service
+    controller = await api.get_service("public/server-apps")
+    
+    # Check initial running apps
+    running_apps_initial = await controller.list_running()
+    
+    # Create a test worker that extends BaseWorker properly
+    class TestWorker(BaseWorker):
+        def __init__(self):
+            super().__init__()
+            self._sessions = {}
+            
+        @property
+        def name(self):
+            return "TestWorker"
+            
+        @property
+        def description(self):
+            return "Test worker for death cleanup testing"
+            
+        @property
+        def supported_types(self):
+            return ["test-app"]
+            
+        @property
+        def use_local_url(self):
+            return True
+            
+        async def start(self, config, context=None):
+            """Start a new worker session."""
+            if isinstance(config, dict):
+                # Convert dict to WorkerConfig for consistency
+                config = WorkerConfig(**config)
+            
+            session_id = config.id
+            session_info = SessionInfo(
+                session_id=session_id,
+                app_id=config.app_id,
+                workspace=config.workspace,
+                client_id=config.client_id,
+                status=SessionStatus.RUNNING,
+                app_type="test-app",
+                created_at="2024-01-01T00:00:00Z",
+                entry_point=config.entry_point,
+            )
+            self._sessions[session_id] = session_info
+            print(f"🚀 TestWorker started session: {session_id}")
+            
+            # Simulate a running app by registering a service
+            # This prevents the app from being stopped immediately
+            import asyncio
+            await asyncio.sleep(0.1)  # Small delay to simulate startup
+            
+            return session_id
+            
+        async def stop(self, session_id, context=None):
+            """Stop a worker session."""
+            if session_id in self._sessions:
+                self._sessions.pop(session_id)
+                print(f"🛑 TestWorker stopped session: {session_id}")
+
+        async def get_logs(self, session_id, type=None, offset=0, limit=None, context=None):
+            """Get logs for a session."""
+            items = [{"type": "info", "content": f"Log from {session_id}"}]
+            if type:
+                items = [item for item in items if item["type"] == type]
+            
+            if limit is not None:
+                items = items[offset:offset + limit]
+            else:
+                items = items[offset:]
+            
+            return {
+                "items": items,
+                "total": 1,
+                "offset": offset,
+                "limit": limit
+            }
+
+    # Connect as a worker client and register the test worker
+    worker_client_id = "test-worker-client-" + random_id()
+    worker_api = await connect_to_server(
+        {"client_id": worker_client_id, "server_url": SERVER_URL, "token": test_user_token}
+    )
+    
+    test_worker = TestWorker()
+    worker_service = test_worker.get_worker_service()
+    
+    await worker_api.register_service(worker_service)
+    
+    # Install a test app that the worker supports
+    test_app_source = """<config lang="json">
+{
+    "name": "Test App for Worker Death",
+    "type": "test-app",
+    "version": "0.1.0",
+    "description": "Test app for worker death cleanup verification"
+}
+</config>
+<script lang="python">
+from hypha_rpc import api
+
+def test_function():
+    return "Hello from test app"
+
+api.export({"test": test_function})
+</script>
+"""
+    
+    app_info = await controller.install(source=test_app_source, overwrite=True, wait_for_service=False)
+    app_id = app_info.get("id") or app_info.get("alias")
+    
+    # Start the app - it should use our test worker
+    session_info = await controller.start(app_id, worker_id=test_worker.service_id, wait_for_service=False)
+    session_id = session_info.get("id")
+    
+    # Verify the session is running
+    running_apps_with_session = await controller.list_running()
+    assert len(running_apps_with_session) > len(running_apps_initial), "Session should be in running list"
+    
+    # Find our session in the running list
+    our_session = None
+    for session in running_apps_with_session:
+        # session might be a dict/object with 'id' field
+        session_id_to_check = session.get('id') if hasattr(session, 'get') else str(session)
+        if session_id in session_id_to_check:
+            our_session = session
+            break
+    
+    assert our_session is not None, f"Our session {session_id} should be in running list"
+    
+    # Now simulate worker death by disconnecting the worker client
+    await worker_api.disconnect()
+    
+    # Wait for cleanup events to be processed
+    await asyncio.sleep(5)
+    
+    # Check running apps - our session should be gone
+    running_apps_after = await controller.list_running()
+    
+    # Verify our session is no longer in the running list
+    session_still_running = False
+    for session in running_apps_after:
+        # session might be a dict/object with 'id' field
+        session_id_to_check = session.get('id') if hasattr(session, 'get') else str(session)
+        if session_id in session_id_to_check:
+            session_still_running = True
+            break
+    
+    assert not session_still_running, f"Session {session_id} should be removed from running list after worker death"
+    
+    # Clean up
+    await controller.uninstall(app_id)
+    await api.disconnect()
+
+@pytest.mark.asyncio
+async def test_use_local_url_functionality(fastapi_server, test_user_token):
+    """Test that built-in workers have use_local_url property set correctly."""
+    # Test that built-in workers have use_local_url=True
+    from hypha.workers.browser import BrowserWorker
+    from hypha.workers.conda import CondaWorker
+    
+    # Test BrowserWorker (built-in worker)
+    browser_worker = BrowserWorker()
+    assert hasattr(browser_worker, 'use_local_url'), "BrowserWorker should have use_local_url property"
+    assert browser_worker.use_local_url == True, "BrowserWorker should have use_local_url=True"
+    
+    # Test that the property is included in worker service registration
+    browser_service = browser_worker.get_worker_service()
+    assert 'use_local_url' in browser_service, "Worker service should include use_local_url"
+    assert browser_service['use_local_url'] == True, "Worker service use_local_url should match property"
+    
+    # Test CondaWorker (can be external worker) - should have use_local_url=False by default
+    conda_worker = CondaWorker()
+    assert hasattr(conda_worker, 'use_local_url'), "CondaWorker should have use_local_url property"
+    assert conda_worker.use_local_url == False, "CondaWorker should have use_local_url=False"
+    
+    conda_service = conda_worker.get_worker_service()
+    assert 'use_local_url' in conda_service, "Worker service should include use_local_url"
+    assert conda_service['use_local_url'] == False, "Worker service use_local_url should match property"
+    
+    # Test the base worker class has the property
+    from hypha.workers.base import BaseWorker
+    
+    class TestWorker(BaseWorker):
+        def __init__(self):
+            super().__init__()
+        
+        @property
+        def name(self):
+            return "test-worker"
+            
+        @property
+        def description(self):
+            return "Test worker"
+            
+        @property
+        def supported_types(self):
+            return ["test"]
+            
+        async def start(self, config, context=None):
+            return "test-session"
+            
+        async def stop(self, session_id, context=None):
+            pass
+
+            
+        async def get_logs(self, session_id, type=None, offset=0, limit=None, context=None):
+            return {"items": [], "total": 0, "offset": offset, "limit": limit}
+
+    
+    test_worker = TestWorker()
+    assert hasattr(test_worker, 'use_local_url'), "BaseWorker should have use_local_url property"
+    assert test_worker.use_local_url == False, "BaseWorker use_local_url should default to False"
+    
+    test_service = test_worker.get_worker_service()
+    assert 'use_local_url' in test_service, "Worker service should include use_local_url"
+    assert test_service['use_local_url'] == False, "Worker service use_local_url should match property"
+    
+    print("✓ use_local_url property is correctly set on all workers")
+    print(f"  BrowserWorker.use_local_url = {browser_worker.use_local_url}")
+    print(f"  CondaWorker.use_local_url = {conda_worker.use_local_url}")
+    print(f"  BaseWorker.use_local_url (default) = {test_worker.use_local_url}")
+    
+    print("✓ Test completed successfully - use_local_url functionality")
+
+
 if __name__ == "__main__":
     pytest.main([__file__, "-v"])
